#include <vtkClipPolyData.h>
#include <vtkFeatureEdges.h>
#include <vtkGeometryFilter.h>
#include <vtkPlane.h>
#include <vtkSmartPointer.h>
#include <vtkSphereSource.h>
#include <vtkXMLPolyDataWriter.h>

#include "vtkAdjustBoundary.h"

int main(int, char *[])
{
  // Create a sphere
  vtkSmartPointer<vtkSphereSource> sphere =
    vtkSmartPointer<vtkSphereSource>::New();
  sphere->SetPhiResolution(20);
  sphere->SetThetaResolution(20);
  sphere->SetCenter(0.0, 0.0, 0.0);
  sphere->SetRadius(5.0);

  // Create a plane to cut the sphere.
  vtkSmartPointer<vtkPlane> plane =
    vtkSmartPointer<vtkPlane>::New();
  plane->SetOrigin(0,0,0);
  plane->SetNormal(1,1,1);

  // Create a cutter.
  vtkSmartPointer<vtkClipPolyData> clipper =
    vtkSmartPointer<vtkClipPolyData>::New();
  clipper->SetClipFunction(plane);
  clipper->SetInputConnection(sphere->GetOutputPort());
  clipper->Update();

  // conert an unstructured grid to a polydata.
  vtkSmartPointer<vtkGeometryFilter> geometryFilter =
    vtkSmartPointer<vtkGeometryFilter>::New();
  geometryFilter->SetInputData(clipper->GetOutput());
  geometryFilter->Update();

  // detects boundaries
  vtkSmartPointer<vtkFeatureEdges> featureEdges =
    vtkSmartPointer<vtkFeatureEdges>::New();
  featureEdges->SetInputConnection(geometryFilter->GetOutputPort());
  featureEdges->BoundaryEdgesOn();
  featureEdges->FeatureEdgesOff();
  featureEdges->ManifoldEdgesOff();
  featureEdges->NonManifoldEdgesOff();
  featureEdges->Update();

  vtkSmartPointer<vtkAdjustBoundary> adjustBoundary =
    vtkSmartPointer<vtkAdjustBoundary>::New();
  adjustBoundary->SetInputConnection(0, geometryFilter->GetOutputPort());
  adjustBoundary->SetInputConnection(1, featureEdges->GetOutputPort());
  adjustBoundary->SetNumberOfIterations(200);
  adjustBoundary->SetEdgeAngle(180.0);
  adjustBoundary->Update();

  // Write the file
  vtkSmartPointer<vtkXMLPolyDataWriter> vtp_writer =
    vtkSmartPointer<vtkXMLPolyDataWriter>::New();
  vtp_writer->SetFileName("out.vtp");
  vtp_writer->SetInputData(adjustBoundary->GetOutput());
  vtp_writer->Write();

  return EXIT_SUCCESS;
}
